@@ -105,7 +105,9 @@
 // kmo_sci_red
 #define SCIENCE                  "SCIENCE"              // RAW
 #define COMBINED_CUBE            "SCI_COMBINED"
+#define COMBINED_COLLAPSED       "SCI_COMBINED_COLL"
 #define RECONSTRUCTED_CUBE       "SCI_RECONSTRUCTED"
+#define RECONSTRUCTED_COLLAPSED  "SCI_RECONSTRUCTED_COLL"
 #define INTERIM_OBJECT_CUBE      "SCI_INTERIM_OBJECT"
 #define INTERIM_OBJECT_SKY       "SCI_INTERIM_SKY"
 // kmo_multi_reconstruct
@@ -159,7 +161,7 @@
 #define STATS                    "STATS"
 // kmo_strip
 #define STRIP                    "STRIP"
-// kmo_sky_tweak
+// kmos_sky_tweak
 #define CUBE_SKY                 "CUBE_SKY"
 #define SKY_TWEAK                "SKY_TWEAK"
 
