@@ -133,8 +133,8 @@ cpl_error_code kmos_combine_frames(
     float               *   pavg_data ;
     float               *   pavg_noise ;
     float               *   pcur_img2 ;
-    int                     nx, ny, nz, ix, iy, iz, rej, new_size;
-    double                  stdev, stderr, tol, tmp_dbl ;
+    int                     nx, ny, nz, ix, iy, iz, rej ;
+    double                  stderr, tol, tmp_dbl ;
     enum combine_status     status = combine_ok;
 
     /* Check inputs */
@@ -204,7 +204,7 @@ cpl_error_code kmos_combine_frames(
     /* Create output noise-image and set to zero */
     if (avg_noise != NULL) {
         *avg_noise = cpl_image_new(nx, ny, CPL_TYPE_FLOAT);
-        cpl_image_multiply_scalar(*avg_noise, 0.0) == CPL_ERROR_NONE;
+        cpl_image_multiply_scalar(*avg_noise, 0.0);
         pavg_noise = cpl_image_get_data_float(*avg_noise);
     }
 
@@ -1493,7 +1493,7 @@ static double kmos_combine_vector(
 {
     kmclipm_vector  *   data ;
     cpl_vector      *   vec_non_rej ;
-    double              out_val, low_thresh, high_thresh, mean, stdev, tol ;
+    double              out_val, low_thresh, high_thresh, mean, stdev ;
     int                 i, j, nz, size, all_rejected, calc_noise ;
 
     /* Check inputs */
@@ -1504,7 +1504,6 @@ static double kmos_combine_vector(
     }
 
     /* Initialise */
-    tol = 1e-6;
     all_rejected = FALSE ;
     calc_noise = FALSE;
 
