@@ -1,5 +1,4 @@
-/* $Id: kmo_priv_std_star.c,v 1.28 2013-07-29 18:10:08 aagudo Exp $
- *
+/*
  * This file is part of the KMOS Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
  *
@@ -18,13 +17,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/*
- * $Author: aagudo $
- * $Date: 2013-07-29 18:10:08 $
- * $Revision: 1.28 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -59,13 +51,9 @@
 #undef __USE_XOPEN
 #endif
 
-/*----------------------------------------------------------------------------*/
-/**
-    @defgroup kmos_priv_std_star     Helper functions for recipe kmo_std_star.
-
-    @{
- */
-/*----------------------------------------------------------------------------*/
+/*-----------------------------------------------------------------------------
+ *                              Defines
+ *----------------------------------------------------------------------------*/
 
 int print_warning_once_stdstar = TRUE;
 int cnter = 0;
@@ -78,252 +66,168 @@ typedef struct {
     int containsObjects;
 } tmpFrameTableStruct;
 
+
+/*-----------------------------------------------------------------------------
+                                Functions prototypes
+ -----------------------------------------------------------------------------*/
+
+
+
+/*----------------------------------------------------------------------------*/
 /**
-    @brief
-        Extracts the spectral and luminosity class of an input string.
+  @defgroup kmos_priv_std_star     Helper functions for recipe kmo_std_star.
+ */
+/*----------------------------------------------------------------------------*/
 
-    @param in_string    The string to examine.
-    @param spec_class   (Output) The identified spectral class.
-    @param lum_class    (Output) The identified luminosity class.
-
-    The input string must be something like 'G2V' odr 'A9III'.
-    The 1st letter can either be: O, B, A, F, G, K.
-    The 2nd letter can range from 0 to 9 (except for K: there only 0 is allowed)
-    The remaning letters specify the lum. class in roman letters:
-    I, II, III, IV, V
-
-    @return             CPL_ERROR_NONE if the classes have been identified
-                        correctly,
-                        CPL_ERROR_ILLEGAL_INPUT otherwise.
-
-    Possible cpl_error_code set in this function:
-
-    @li CPL_ERROR_NULL_INPUT     if any of the inputs is NULL.
-    @li CPL_ERROR_ILLEGAL_INPUT  if they provided star type is wrong.
-*/
-cpl_error_code kmo_get_spec_type(const char *in_string,
-                                 char *spec_class,
-                                 char *lum_class)
-{
-    cpl_error_code  ret_err         = CPL_ERROR_NONE;
-    char            tmp_str1[5],
-                    tmp_str2[5],
-                    *tmp_in_string  = NULL;
-
-    KMO_TRY
-    {
-        KMO_TRY_ASSURE((in_string != NULL) &&
-                       (spec_class != NULL) &&
-                       (lum_class != NULL),
-                       CPL_ERROR_NULL_INPUT,
-                       "No input data is provided!");
-
-        // make upper case
-        KMO_TRY_EXIT_IF_NULL(
-            tmp_in_string = cpl_sprintf("%s", in_string));
-        kmo_strupper(tmp_in_string);
-
-        // get spec_class
-        strncpy(spec_class, tmp_in_string, 2);
-        spec_class[2] = '\0';
-
-        // test spec_class
-        strncpy(tmp_str1, spec_class, 1);
-        tmp_str1[1] = '\0';
-
-        KMO_TRY_ASSURE((strcmp(tmp_str1, "O") == 0) ||
-                       (strcmp(tmp_str1, "B") == 0) ||
-                       (strcmp(tmp_str1, "A") == 0) ||
-                       (strcmp(tmp_str1, "F") == 0) ||
-                       (strcmp(tmp_str1, "G") == 0) ||
-                       (strcmp(tmp_str1, "K") == 0),
-                       CPL_ERROR_ILLEGAL_INPUT,
-                       "Only O, B, A, F, G, K stars are supported!");
-
-        strcpy(tmp_str2, spec_class + 1);
-
-        if (strcmp(tmp_str1, "K") == 0) {
-            // K
-            KMO_TRY_ASSURE(strcmp(tmp_str2, "0") == 0,
-                           CPL_ERROR_ILLEGAL_INPUT,
-                           "For K the 2nd letter must be 0!");
-        } else if (strcmp(tmp_str1, "O") == 0) {
-            // O
-            KMO_TRY_ASSURE((strcmp(tmp_str2, "1") == 0) ||
-                           (strcmp(tmp_str2, "2") == 0) ||
-                           (strcmp(tmp_str2, "3") == 0) ||
-                           (strcmp(tmp_str2, "4") == 0) ||
-                           (strcmp(tmp_str2, "5") == 0) ||
-                           (strcmp(tmp_str2, "6") == 0) ||
-                           (strcmp(tmp_str2, "7") == 0) ||
-                           (strcmp(tmp_str2, "8") == 0) ||
-                           (strcmp(tmp_str2, "9") == 0),
-                           CPL_ERROR_ILLEGAL_INPUT,
-                           "For O stars the 2nd letter must be a single "
-                           "digit and not zero!");
-        } else {
-            // B, A, F, G
-            KMO_TRY_ASSURE((strcmp(tmp_str2, "0") == 0) ||
-                           (strcmp(tmp_str2, "1") == 0) ||
-                           (strcmp(tmp_str2, "2") == 0) ||
-                           (strcmp(tmp_str2, "3") == 0) ||
-                           (strcmp(tmp_str2, "4") == 0) ||
-                           (strcmp(tmp_str2, "5") == 0) ||
-                           (strcmp(tmp_str2, "6") == 0) ||
-                           (strcmp(tmp_str2, "7") == 0) ||
-                           (strcmp(tmp_str2, "8") == 0) ||
-                           (strcmp(tmp_str2, "9") == 0),
-                           CPL_ERROR_ILLEGAL_INPUT,
-                           "The 2nd letter must be a single digit!");
-        }
-
-        // get lum_class
-        strcpy(lum_class, tmp_in_string + 2);
-
-        KMO_TRY_ASSURE((strcmp(lum_class, "I") == 0) ||
-                       (strcmp(lum_class, "II") == 0) ||
-                       (strcmp(lum_class, "III") == 0) ||
-                       (strcmp(lum_class, "IV") == 0) ||
-                       (strcmp(lum_class, "V") == 0),
-                       CPL_ERROR_ILLEGAL_INPUT,
-                       "The luminosity must be between I and IV!");
-    }
-    KMO_CATCH
-    {
-//        KMO_CATCH_MSG();
-        ret_err = cpl_error_get_code();
-        spec_class = "\0";
-        lum_class = "\0";
-    }
-
-    cpl_free(tmp_in_string); tmp_in_string = NULL;
-
-    return ret_err;
-}
+/**@{*/
 
+/*----------------------------------------------------------------------------*/
 /**
-    @brief
-        Gets the temperature of a specified star type.
+  @brief    Get the star temperature
+  @param    fset        The frame set
+  @param    spec_type   The user specified spec type (or "")
+  @param    star_type   The computed star type
 
-    @param spec_table   The table containing the luminosities.
-    @param spec_class   The spectral class.
-    @param lum_class    The luminosity class.
-
-    spec_class and lum_class should be generated with kmo_get_spec_type().
-
-    @return The temperature of the specified star.
-
-    Possible cpl_error_code set in this function:
-
-    @li CPL_ERROR_NULL_INPUT     if any of the inputs is NULL.
-    @li CPL_ERROR_ILLEGAL_INPUT  if they provided classes are wrong.
+  The input string must be something like 'G2V' odr 'A9III'.
+  The 1st letter can either be: O, B, A, F, G, K.
+  The 2nd letter can range from 0 to 9 
+  The remaning letters specify the lum. class in roman letters:
+  I, II, III, IV, V
+  @return   the temperature or -1.0 if it could not be derived
+  Possible cpl_error_code set in this function:
+      @li CPL_ERROR_NULL_INPUT     if any of the inputs is NULL.
+      @li CPL_ERROR_ILLEGAL_INPUT  if they provided star type is wrong.
 */
-double kmo_get_temperature(cpl_table *spec_table,
-                          const char *spec_class,
-                          const char *lum_class)
+/*----------------------------------------------------------------------------*/
+double kmos_get_temperature(
+        cpl_frameset        *   fset,
+        const char          *   spec_type,
+        char                *   star_type)
 {
-    double  ret_val     = 0.0;
+    cpl_frame           *   std_frame ;
+    cpl_propertylist    *   main_h ;
+    char                *   spec_type_loc ;
+    double                  star_temperature ;
+    cpl_table           *   spec_table ;
+    char                    lum_class[8] ;
+    int                     ind_row, d ;
 
-    char    tmp_str[5],
-            tmp_str1[5],
-            tmp_str2[5];
-
-    int     ind_row     = 0,
-            d           = 0;
-
-    KMO_TRY
-    {
-        KMO_TRY_ASSURE((spec_table != NULL) &&
-                       (spec_class != NULL) &&
-                       (lum_class != NULL),
-                       CPL_ERROR_NULL_INPUT,
-                       "No input data is provided!");
-
-        KMO_TRY_ASSURE((cpl_table_get_ncol(spec_table) == 5) &&
-                       (cpl_table_get_nrow(spec_table) == 50),
-                       CPL_ERROR_ILLEGAL_INPUT,
-                       "SPEC_TYPE_LOOKUP must have 5 columns and 50 rows!");
-
-        // get spec_class
-        strncpy(tmp_str, spec_class, 2);
-        tmp_str[2] = '\0';
-
-        // test spec_class
-        strncpy(tmp_str1, tmp_str, 1);
-        tmp_str1[1] = '\0';
-
-        kmo_strupper(tmp_str1);
-
-        if (strcmp(tmp_str1, "O") == 0) {
-            ind_row = -1;
-        } else if (strcmp(tmp_str1, "B") == 0) {
-            ind_row = 9;
-        } else if (strcmp(tmp_str1, "A") == 0) {
-            ind_row = 19;
-        } else if (strcmp(tmp_str1, "F") == 0) {
-            ind_row = 29;
-        } else if (strcmp(tmp_str1, "G") == 0) {
-            ind_row = 39;
-        } else if (strcmp(tmp_str1, "K") == 0) {
-            ind_row = 49;
-        } else {
-            KMO_TRY_ASSURE(1 == 0,
-                           CPL_ERROR_ILLEGAL_INPUT,
-                           "Only O, B, A, F, G, K stars are supported!");
-        }
-
-        strcpy(tmp_str2, tmp_str + 1);
-
-        if (strcmp(tmp_str1, "K") == 0) {
-            // K
-            KMO_TRY_ASSURE(strcmp(tmp_str2, "0") == 0,
-                           CPL_ERROR_ILLEGAL_INPUT,
-                           "For K stars the 2nd letter must be 0!");
-        } else {
-            // O, B, A, F, G
-            if (strcmp(tmp_str1, "O") == 0) {
-                KMO_TRY_ASSURE(strcmp(tmp_str2, "0") != 0,
-                               CPL_ERROR_ILLEGAL_INPUT,
-                               "For O stars only classes from "
-                               "1 to 9 are known!");
-            }
-
-            if (strcmp(tmp_str2, "0") == 0) {
-                ;
-            } else if (strcmp(tmp_str2, "1") == 0) {
-                ind_row += 1;
-            } else if (strcmp(tmp_str2, "2") == 0) {
-                ind_row += 2;
-            } else if (strcmp(tmp_str2, "3") == 0) {
-                ind_row += 3;
-            } else if (strcmp(tmp_str2, "4") == 0) {
-                ind_row += 4;
-            } else if (strcmp(tmp_str2, "5") == 0) {
-                ind_row += 5;
-            } else if (strcmp(tmp_str2, "6") == 0) {
-                ind_row += 6;
-            } else if (strcmp(tmp_str2, "7") == 0) {
-                ind_row += 7;
-            } else if (strcmp(tmp_str2, "8") == 0) {
-                ind_row += 8;
-            } else if (strcmp(tmp_str2, "9") == 0) {
-                ind_row += 9;
-            } else {
-                KMO_TRY_ASSURE(1 == 0,
-                               CPL_ERROR_ILLEGAL_INPUT,
-                               "The 2nd letter must be a single digit!");
-            }
-        }
-
-        ret_val =  cpl_table_get(spec_table, lum_class, ind_row, &d);
-    }
-    KMO_CATCH
-    {
-        KMO_CATCH_MSG();
-        ret_val = 0.0;
+    /* Check Entries */
+    if (fset == NULL || spec_type == NULL || star_type == NULL) {
+        cpl_msg_error(__func__, "NULL inputs") ;
+        cpl_error_set(__func__, CPL_ERROR_NULL_INPUT) ;
+        return -1.0 ;
     }
 
-    return ret_val * 1000.;
+    /* If the passed spec_type is empty, use the header */
+    if (!strcmp(spec_type, "")) {
+        std_frame = kmo_dfs_get_frame(fset, STD);
+        main_h=kmclipm_propertylist_load(cpl_frame_get_filename(std_frame),0);
+
+        // check for startype-keyword
+        if ((cpl_propertylist_has(main_h, STDSTAR_TYPE)) &&
+            (cpl_propertylist_get_type(main_h, STDSTAR_TYPE)==CPL_TYPE_STRING)){
+            spec_type_loc = cpl_strdup(
+                    cpl_propertylist_get_string(main_h, STDSTAR_TYPE));
+        } else {
+            spec_type_loc = NULL ;
+        }
+        cpl_propertylist_delete(main_h) ;
+    } else {
+        spec_type_loc = cpl_strdup(spec_type) ;
+    }
+
+    /* No Spectral type */
+    if (spec_type_loc == NULL) {
+        cpl_msg_error(__func__, "Cannot determine the spectral type") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return -1.0 ;
+    }
+
+    /* Upper case */
+    kmo_strupper(spec_type_loc);
+
+    /* Check the spectral type */
+    if (spec_type_loc[0] != 'O' && spec_type_loc[0] != 'B' &&
+            spec_type_loc[0] != 'A' && spec_type_loc[0] != 'F' &&
+            spec_type_loc[0] != 'G' && spec_type_loc[0] != 'K') {
+        cpl_free(spec_type_loc) ;
+        cpl_msg_error(__func__, "O, B, A, F, G, K stars are supported") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return -1.0 ;
+    }
+    *star_type = spec_type_loc[0] ;
+
+    if (spec_type_loc[0] == 'K' && spec_type_loc[1] != '0') {
+        cpl_free(spec_type_loc) ;
+        cpl_msg_error(__func__, "For K the 2nd letter must be 0") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return -1.0 ;
+    }
+    if (spec_type_loc[0] == 'O' && spec_type_loc[1] == '0') {
+        cpl_free(spec_type_loc) ;
+        cpl_msg_error(__func__, "For O the 2nd letter cannot be 0") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return -1.0 ;
+    }
+    if (spec_type_loc[1] != '0' && spec_type_loc[1] != '1' &&
+            spec_type_loc[1] != '2' && spec_type_loc[1] != '3' &&
+            spec_type_loc[1] != '4' && spec_type_loc[1] != '5' &&
+            spec_type_loc[1] != '6' && spec_type_loc[1] != '7' &&
+            spec_type_loc[1] != '8' && spec_type_loc[1] != '9') {
+        cpl_free(spec_type_loc) ;
+        cpl_msg_error(__func__, "The 2nd letter must be a digit 0-9") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return -1.0 ;
+    }
+
+    /* Load the SPEC_TYPE_LOOKUP table */
+    spec_table = kmo_dfs_load_table(fset, SPEC_TYPE_LOOKUP, 1, 0);
+    if (spec_table == NULL) {
+        cpl_free(spec_type_loc) ;
+        cpl_msg_error(__func__, "Cannot load the spec type lookup table") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return -1.0 ;
+    }
+    if (cpl_table_get_ncol(spec_table) != 5 || 
+            cpl_table_get_nrow(spec_table) != 50) {
+        cpl_free(spec_type_loc) ;
+        cpl_msg_error(__func__, "Spec type lookup table bad dimensions") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return -1.0 ;
+    }
+
+    /* Get Row index */
+    ind_row = 0 ;
+    if (spec_type_loc[0] == 'O')    ind_row = -1;
+    if (spec_type_loc[0] == 'B')    ind_row = 9;
+    if (spec_type_loc[0] == 'A')    ind_row = 19;
+    if (spec_type_loc[0] == 'F')    ind_row = 29;
+    if (spec_type_loc[0] == 'G')    ind_row = 39;
+    if (spec_type_loc[0] == 'K')    ind_row = 49;
+
+    if (spec_type_loc[1] == '1')    ind_row += 1; 
+    if (spec_type_loc[1] == '2')    ind_row += 2; 
+    if (spec_type_loc[1] == '3')    ind_row += 3; 
+    if (spec_type_loc[1] == '4')    ind_row += 4; 
+    if (spec_type_loc[1] == '5')    ind_row += 5; 
+    if (spec_type_loc[1] == '6')    ind_row += 6; 
+    if (spec_type_loc[1] == '7')    ind_row += 7; 
+    if (spec_type_loc[1] == '8')    ind_row += 8; 
+    if (spec_type_loc[1] == '9')    ind_row += 9; 
+
+    /* Check lum_class */
+    strcpy(lum_class, spec_type_loc + 2);
+
+    /* Get the temperature from the table */
+    star_temperature = 1000 * cpl_table_get(spec_table, lum_class, ind_row, &d);
+    cpl_free(spec_type_loc) ;
+
+    if (cpl_error_get_code() != CPL_ERROR_NONE) {
+        cpl_error_reset();
+        star_temperature = -1.0 ;
+    }
+    return star_temperature ;
 }
 
 /**
@@ -860,30 +764,28 @@ cpl_error_code kmo_remove_line(cpl_vector *spectrum,
     return ret_err;
 }
 
+/*----------------------------------------------------------------------------*/
 /**
-    @brief
-        Calculates the counts/sec of a spectrum.
-
-    @param data        The spectral data.
-    @param filter_id   The filter used.
-    @param crpix       CRPIX1.
-    @param crval       CRVAL1.
-    @param cdelt       CDELT1.
-
-    @return The counts/sec in a certain interval of the input spectrum.
-
-    Possible cpl_error_code set in this function:
-
-    @li CPL_ERROR_NULL_INPUT     if any of the inputs is NULL.
-    @li CPL_ERROR_ILLEGAL_INPUT  if an invalid filter is provided.
+  @brief Calculates the counts/sec of a spectrum.
+  @param data        The spectral data.
+  @param filter_id   The filter used.
+  @param crpix       CRPIX1.
+  @param crval       CRVAL1.
+  @param cdelt       CDELT1.
+  @return The counts/sec in a certain interval of the input spectrum.
+  Possible cpl_error_code set in this function:
+      @li CPL_ERROR_NULL_INPUT     if any of the inputs is NULL.
+      @li CPL_ERROR_ILLEGAL_INPUT  if an invalid filter is provided.
 */
-cpl_error_code kmo_calc_counts(const cpl_vector *data,
-                               const char *filter_id,
-                               double crpix,
-                               double crval,
-                               double cdelt,
-                               double *counts_band1,
-                               double *counts_band2)
+/*----------------------------------------------------------------------------*/
+cpl_error_code kmo_calc_counts(
+        const cpl_vector    *   data,
+        const char          *   filter_id,
+        double                  crpix,
+        double                  crval,
+        double                  cdelt,
+        double              *   counts_band1,
+        double              *   counts_band2)
 {
     cpl_vector      *data_l     = NULL;
     const double    *pdata      = NULL;
@@ -891,91 +793,67 @@ cpl_error_code kmo_calc_counts(const cpl_vector *data,
                     min_l       = 0.0,
                     max_l       = 0.0;
     int             size        = 0,
-                    i           = 0;
-    cpl_error_code  ret_err     = CPL_ERROR_NONE;
+                   i           = 0;
+ 
+    /* Check Entries */
+    if (data==NULL||filter_id==NULL||counts_band1==NULL||counts_band2==NULL) {
+        cpl_msg_error(__func__, "NULL inputs") ;
+        cpl_error_set(__func__, CPL_ERROR_NULL_INPUT) ;
+        return CPL_ERROR_NULL_INPUT ;
+    }
+    if (strcmp(filter_id, "K") && strcmp(filter_id, "H") &&
+            strcmp(filter_id, "HK") && strcmp(filter_id, "IZ") &&
+            strcmp(filter_id, "YJ")) {
+        cpl_msg_error(__func__, "Wrong filter provided") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return CPL_ERROR_ILLEGAL_INPUT ;
+    }
 
-    KMO_TRY
-    {
-        KMO_TRY_CHECK_ERROR_STATE();
+    /* Initialise */
+    *counts_band1 = *counts_band2 = 0.0 ;
 
-        KMO_TRY_ASSURE((data != NULL) &&
-                       (filter_id != NULL) &&
-                       (counts_band1 != NULL) &&
-                       (counts_band2 != NULL),
-                       CPL_ERROR_NULL_INPUT,
-                       "Not all input data is provided!");
+    /* Calculate lambda range */
+    if (!strcmp(filter_id, "K")) {
+        min_l = LAMBDA_MIN_K;
+        max_l = LAMBDA_MAX_K;
+    } else if (!strcmp(filter_id, "H") || !strcmp(filter_id, "HK")) {
+        min_l = LAMBDA_MIN_H;
+        max_l = LAMBDA_MAX_H;
+    } else if (!strcmp(filter_id, "IZ")) {
+        min_l = LAMBDA_MIN_Z;
+        max_l = LAMBDA_MAX_Z;
+    } else if (!strcmp(filter_id, "YJ")) {
+        min_l = LAMBDA_MIN_J;
+        max_l = LAMBDA_MAX_J;
+    }
 
-        KMO_TRY_ASSURE((strcmp(filter_id, "K") == 0) ||
-                       (strcmp(filter_id, "H") == 0) ||
-                       (strcmp(filter_id, "HK") == 0) ||
-                       (strcmp(filter_id, "IZ") == 0) ||
-                       (strcmp(filter_id, "YJ") == 0),
-                       CPL_ERROR_ILLEGAL_INPUT,
-                       "Wrong filter provided (%s)!", filter_id);
+    /* Setup vectors & data pointers to them */
+    size = cpl_vector_get_size(data);
+    pdata = cpl_vector_get_data_const(data);
+    data_l = kmo_create_lambda_vec(size, crpix, crval, cdelt);
+    pdata_l = cpl_vector_get_data(data_l);
 
-        // calculate lambda range
-        if (strcmp(filter_id, "K") == 0) {
-            min_l = LAMBDA_MIN_K;
-            max_l = LAMBDA_MAX_K;
-        } else if ((strcmp(filter_id, "H") == 0) || (strcmp(filter_id, "HK") == 0)) {
-            min_l = LAMBDA_MIN_H;
-            max_l = LAMBDA_MAX_H;
-        } else if (strcmp(filter_id, "IZ") == 0) {
-            min_l = LAMBDA_MIN_Z;
-            max_l = LAMBDA_MAX_Z;
-        } else if (strcmp(filter_id, "YJ") == 0) {
-            min_l = LAMBDA_MIN_J;
-            max_l = LAMBDA_MAX_J;
-        }
+    /* Calculate counts for 1st band */
+    for (i = 0; i < size; i++) {
+        if ((pdata_l[i] >= min_l) && (pdata_l[i] <= max_l) &&
+            !kmclipm_is_nan_or_inf(pdata[i])) 
+            *counts_band1 += pdata[i];
+    }
 
-        // setup vectors & data pointers to them
-        size = cpl_vector_get_size(data);
-        KMO_TRY_EXIT_IF_NULL(
-            pdata = cpl_vector_get_data_const(data));
-        KMO_TRY_EXIT_IF_NULL(
-            data_l = kmo_create_lambda_vec(size, crpix, crval, cdelt));
-        KMO_TRY_EXIT_IF_NULL(
-            pdata_l = cpl_vector_get_data(data_l));
+    /* Calculate counts for 2nd band */
+    if (strcmp(filter_id, "HK") == 0) {
+        min_l = LAMBDA_MIN_K;
+        max_l = LAMBDA_MAX_K;
 
-        // calculate counts for 1st band
+        /* Calculate counts for second band */
         for (i = 0; i < size; i++) {
             if ((pdata_l[i] >= min_l) && (pdata_l[i] <= max_l) &&
                 !kmclipm_is_nan_or_inf(pdata[i]))
-            {
-                *counts_band1 += pdata[i];
-            }
-        }
-
-        // calculate counts for 2nd band
-        if (strcmp(filter_id, "HK") == 0) {
-            min_l = LAMBDA_MIN_K;
-            max_l = LAMBDA_MAX_K;
-
-            // calculate counts for second band
-            for (i = 0; i < size; i++) {
-                if ((pdata_l[i] >= min_l) && (pdata_l[i] <= max_l) &&
-                    !kmclipm_is_nan_or_inf(pdata[i]))
-                {
-                    *counts_band2 += pdata[i];
-                }
-            }
+                *counts_band2 += pdata[i];
         }
     }
-    KMO_CATCH
-    {
-        KMO_CATCH_MSG();
-        if (counts_band1 != NULL) {
-            *counts_band1 = 0.0;
-        }
-        if (counts_band2 != NULL) {
-            *counts_band2 = 0.0;
-        }
-        ret_err = cpl_error_get_code();
-    }
-
-    cpl_vector_delete(data_l); data_l = NULL;
-
-    return ret_err;
+    cpl_vector_delete(data_l); 
+    return CPL_ERROR_NONE ;
 }
 
 /**
