@@ -274,7 +274,7 @@ static int kmos_dark_destroy(cpl_plugin *plugin)
 static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
 {
     const cpl_parameter *   par ;
-    int                     nx, ny, nz, ne, ndit ;
+    int                     nx, ny, nz, next, ndit ;
     const char          *   cmethod ;
     const char          *   my_method ;
     cpl_frame           *   frame ;
@@ -304,6 +304,10 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
         return -1 ;
     }
 
+    /* Inistialise */
+    nx = ny = next = -1 ;
+    exptime = -1.0 ;
+
     /* Get Parameters */
     par=cpl_parameterlist_find_const(parlist, "kmos.kmos_dark.pos_bad_pix_rej");
     pos_bad_pix_rej = cpl_parameter_get_double(par);
@@ -322,7 +326,7 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
     }
 
     /* Check the inputs consistency */
-    if (kmos_dark_check_inputs(frameset, &nx, &ny, &ndit, &ne, &exptime) != 1) {
+    if (kmos_dark_check_inputs(frameset, &nx, &ny, &ndit, &next,&exptime) != 1){
         cpl_msg_error(__func__, "Input frameset is not consistent") ;
         cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
         return -1 ;
@@ -366,7 +370,7 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
             cpl_func);
 
     /* Loop on detectors */
-    for (i = 1; i <= ne ; i++) {
+    for (i = 1; i <= next ; i++) {
         cpl_msg_info(cpl_func, "Processing detector No. %d", i);
         cpl_msg_indent_more() ;
 
@@ -571,7 +575,7 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
   @param    nx          [out] images x size
   @param    ny          [out] images x size
   @param    ndit        [out] NDIT
-  @param    ne          [out] Nb extensions
+  @param    next        [out] Nb extensions
   @param    exptime_on  [out] exptime
   @return   1 if consistent, 0 if not, -1 in error case
  */
@@ -581,7 +585,7 @@ static int kmos_dark_check_inputs(
         int                 *   nx,
         int                 *   ny,
         int                 *   ndit,
-        int                 *   ne,
+        int                 *   next,
         double              *   exptime)
 {
     cpl_frame           *   frame ;
@@ -593,7 +597,7 @@ static int kmos_dark_check_inputs(
 
     /* Check Entries */
     if (nx == NULL || ny == NULL || frameset == NULL || exptime == NULL ||
-            ndit == NULL || ne == NULL) {
+            ndit == NULL || next == NULL) {
         return -1;
     }
 
@@ -623,9 +627,9 @@ static int kmos_dark_check_inputs(
         if (i == 0) {
             *exptime = exptime_cur ;
             *ndit = ndit_cur ;
-            *ne = ne_cur ;
+            *next = ne_cur ;
         } else {
-            if (ndit_cur != *ndit || ne_cur != *ne || 
+            if (ndit_cur != *ndit || ne_cur != *next || 
                     fabs(exptime_cur-(*exptime)) >1e-3) {
                 cpl_msg_error(__func__, "Header keywords are inconsistent") ;
                 return -1 ;
@@ -642,7 +646,7 @@ static int kmos_dark_check_inputs(
     frame = kmo_dfs_get_frame(frameset, DARK);
     while (frame != NULL) {
         /* Loop on extensions */
-        for (j=1 ; j<=*ne ; j++) {
+        for (j=1 ; j<=*next ; j++) {
             /* Read extension header */
             eh = cpl_propertylist_load(cpl_frame_get_filename(frame), j);
             nx_cur = kmos_pfits_get_naxis1(eh) ;
