@@ -55,7 +55,7 @@
  *----------------------------------------------------------------------------*/
 
 static int kmos_wave_cal_check_inputs(cpl_frameset *, int *, int *, int *, 
-        double *, int *, enum lampConfiguration *);
+        double *, int *, lampConfiguration *);
 
 static int kmos_wave_cal_create(cpl_plugin *);
 static int kmos_wave_cal_exec(cpl_plugin *);
@@ -114,6 +114,9 @@ static char kmos_wave_cal_description[] =
 "multiple products with the same category are produced, they will be numered\n"
 "consecutively starting from 0.\n"
 "\n"
+"--lines_estimation\n"
+"If set to TRUE, the lines estimation method is used\n"
+"\n"
 "----------------------------------------------------------------------------\n"
 "Input files:\n"
 "\n"
@@ -223,6 +226,14 @@ static int kmos_wave_cal_create(cpl_plugin *plugin)
     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
     cpl_parameterlist_append(recipe->parameters, p);
 
+    /* --lines_estimation */
+    p = cpl_parameter_new_value("kmos.kmos_wave_cal.lines_estimation",
+            CPL_TYPE_BOOL, "Trigger lines estimation method",
+            "kmos.kmos_wave_cal", FALSE);
+    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "lines_estimation");
+    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
+    cpl_parameterlist_append(recipe->parameters, p);
+
     /* Add parameters for band-definition */
     kmos_band_pars_create(recipe->parameters, "kmos.kmos_wave_cal");
 
@@ -301,17 +312,15 @@ static int kmos_wave_cal_destroy(cpl_plugin *plugin)
 /*----------------------------------------------------------------------------*/
 static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
 {
-    char                    *   str_line_estimate_method ;
-    int                         line_estimate_method ;
     const cpl_parameter     *   par ;
     int                         suppress_extension, fit_order_par, fit_order ;
-    int                         nx, ny, ne, reduce_det ;
+    int                         nx, ny, next, reduce_det, lines_estimation ;
     double                      exptime, gain, angle_found, reduce_angle ;
     cpl_frame               *   frame ; 
     cpl_propertylist        *   mh_on ;
     cpl_propertylist        *   plist ;
     char                    *   suffix ;
-    enum lampConfiguration      lamp_config;
+    lampConfiguration           lamp_config;
     char                    **  filter_ids ;
     int                     *   angles_array ;
     int                         nb_angles ;
@@ -354,15 +363,6 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     const char              *   tmp_str ;
     cpl_error_code              err ;
 
-
-    /* Lines method TODO */
-    str_line_estimate_method = getenv("KMO_WAVE_LINE_ESTIMATE");
-    if (str_line_estimate_method != NULL) {
-        line_estimate_method = atoi(str_line_estimate_method);
-    } else {
-        line_estimate_method = 2 ;
-    }
-
     /* Check entries */
     if (parlist == NULL || frameset == NULL) {
         cpl_msg_error(__func__, "Null Inputs") ;
@@ -373,6 +373,9 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     /* Get Parameters */
     par = cpl_parameterlist_find_const(parlist, "kmos.kmos_wave_cal.order");
     fit_order_par = cpl_parameter_get_int(par);
+    par=cpl_parameterlist_find_const(parlist, 
+            "kmos.kmos_wave_cal.lines_estimation");
+    lines_estimation = cpl_parameter_get_bool(par);
     par=cpl_parameterlist_find_const(parlist, 
             "kmos.kmos_wave_cal.suppress_extension");
     suppress_extension = cpl_parameter_get_bool(par);
@@ -403,7 +406,7 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     }
 
     /* Check the inputs consistency */
-    if (kmos_wave_cal_check_inputs(frameset, &nx, &ny, &ne, &exptime,
+    if (kmos_wave_cal_check_inputs(frameset, &nx, &ny, &next, &exptime,
                 &non_dest_rom, &lamp_config) != 1) {
         cpl_msg_error(__func__, "Input frameset is not consistent") ;
         cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
@@ -418,7 +421,7 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     /* filter/grating can be different for each detector */
     frame = kmo_dfs_get_frame(frameset, ARC_ON);
     mh_on = cpl_propertylist_load(cpl_frame_get_filename(frame), 0);
-    filter_ids =  kmo_get_filter_setup(mh_on, ne, TRUE) ;
+    filter_ids =  kmo_get_filter_setup(mh_on, next, TRUE) ;
     cpl_propertylist_delete(mh_on);
     if (filter_ids == NULL) {
         cpl_free(suffix);
@@ -431,7 +434,7 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     if ((angles_array = kmos_get_angles(frameset, &nb_angles, ARC_ON)) == NULL){
         cpl_msg_error(__func__, "Cannot get Angles informations") ;
         cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
-        for (i = 0; i < ne ; i++) cpl_free(filter_ids[i]);
+        for (i = 0; i < next ; i++) cpl_free(filter_ids[i]);
         cpl_free(filter_ids);
         cpl_free(suffix);
         return -1 ;
@@ -444,7 +447,7 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     if (strcmp(filter_ids[0], tmp_str) != 0) {
         cpl_msg_error(__func__, "Wrong ARC_LIST filter") ;
         cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
-        for (i = 0; i < ne ; i++) cpl_free(filter_ids[i]);
+        for (i = 0; i < next ; i++) cpl_free(filter_ids[i]);
         cpl_free(filter_ids);
         cpl_free(angles_array);
         cpl_propertylist_delete(plist); 
@@ -454,15 +457,16 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
 
     /* Load the lines as a CPL table */
     arclines = kmo_dfs_load_table(frameset, ARC_LIST, 1, 0);
-    lines = kmo_get_lines(arclines, lamp_config);
+    lines = kmos_get_lines(arclines, lamp_config);
     cpl_table_delete(arclines);
     /* TODO : check not null */
     cpl_msg_info(__func__, "Arc lines: %lld", cpl_bivector_get_size(lines));
 
     /* Load REFLINES */
-    if (line_estimate_method == 2) {
+    if (lines_estimation == 0) {
         reflines = kmo_dfs_load_table(frameset, REF_LINES, 1, 0);
-        /* TODO : check not null */
+    } else {
+        reflines = NULL ;
     }
 
     /* Check which IFUs are active for all FLAT frames */
@@ -474,21 +478,23 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     /* make sure no reconstruction lookup table (LUT) is used */
     if (getenv("KMCLIPM_PRIV_RECONSTRUCT_LUT_MODE") != NULL) {
         last_env = getenv("KMCLIPM_PRIV_RECONSTRUCT_LUT_MODE");
+    } else {
+        last_env = NULL ;
     }
     setenv("KMCLIPM_PRIV_RECONSTRUCT_LUT_MODE","NONE",1);
 
     /* the frames have to be stored temporarily because the QC params */
     /* for the main header are calculated per detector. So they can be */
     /* stored only when all detectors are processed */
-    stored_lcal = (cpl_image**)cpl_calloc(ne * nb_angles, sizeof(cpl_image*));
-    stored_det_img = (cpl_image**)cpl_calloc(ne * nb_angles,sizeof(cpl_image*));
-    stored_sub_headers_lcal = (cpl_propertylist**)cpl_calloc(ne * nb_angles,
+    stored_lcal = (cpl_image**)cpl_calloc(next * nb_angles, sizeof(cpl_image*));
+    stored_det_img = (cpl_image**)cpl_calloc(next*nb_angles,sizeof(cpl_image*));
+    stored_sub_headers_lcal = (cpl_propertylist**)cpl_calloc(next * nb_angles,
             sizeof(cpl_propertylist*));
-    stored_sub_headers_det_img = (cpl_propertylist**)cpl_calloc(ne * nb_angles,
+    stored_sub_headers_det_img = (cpl_propertylist**)cpl_calloc(next*nb_angles,
             sizeof(cpl_propertylist*));
-    stored_qc_arc_sat = (int*)cpl_calloc(ne, nb_angles * sizeof(int));
-    stored_qc_ar_eff=(double*)cpl_calloc(ne, nb_angles * sizeof(double));
-    stored_qc_ne_eff=(double*)cpl_calloc(ne, nb_angles * sizeof(double));
+    stored_qc_arc_sat = (int*)cpl_calloc(next, nb_angles * sizeof(int));
+    stored_qc_ar_eff=(double*)cpl_calloc(next, nb_angles * sizeof(double));
+    stored_qc_ne_eff=(double*)cpl_calloc(next, nb_angles * sizeof(double));
 
     /* Loop all Rotator Angles and Detectors  */
     for (a = 0; a < nb_angles; a++) {
@@ -498,8 +504,8 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
         cpl_msg_info(__func__, "Processing rotator angle %d -> %d degree", 
                 a, angles_array[a]);
         cpl_msg_indent_more();
-        for (i = 1; i <= ne ; i++) {
-            /* Compute only one detector */
+        for (i = 1; i <= next ; i++) {
+            /* Compute only one detetor */
             if (reduce_det != 0 && i != reduce_det) continue ;
 
             cpl_msg_info(__func__,"Processing detector No. %d", i);
@@ -540,7 +546,7 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
             /* Get ARC_ON frame and Load it */
             frame = kmos_get_angle_frame(frameset, angles_array[a], ARC_ON);
             det_lamp_on = kmo_dfs_load_image_frame(frame,i,FALSE, TRUE,&nr_sat);
-            int sx = a * ne + (i - 1);
+            int sx = a * next + (i - 1);
 
             /* Count saturated pixels for each detector */
             if (non_dest_rom)   
@@ -564,6 +570,19 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
                     (double)angles_array[a], FALSE, NULL, &angle_found, -1,0,0);
             ycal = kmo_dfs_load_cal_image(frameset, YCAL, i, 0, 
                     (double)angles_array[a], FALSE, NULL, &angle_found, -1,0,0);
+            if (xcal == NULL || ycal == NULL) {
+                /* Missing calibration for this detector */
+                cpl_error_reset() ;
+                stored_det_img[sx] = NULL ;
+                stored_lcal[sx] = cpl_image_new(nx, ny, CPL_TYPE_FLOAT);
+                kmo_image_fill(stored_lcal[sx], 0.0);
+                if (xcal != NULL) cpl_image_delete(xcal) ;
+                if (ycal != NULL) cpl_image_delete(ycal) ;
+                cpl_image_delete(det_lamp_on_copy) ;
+                cpl_image_delete(det_lamp_on) ;
+                cpl_image_delete(det_lamp_off) ;
+                continue ;
+            }
 
             /* Derive BPM from XCAL : NaNs to 0, Others to 1  */
             bad_pix_mask = cpl_image_duplicate(xcal);
@@ -579,11 +598,11 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
             }
 
             /* Compute wavelength calibration */
-            err = kmos_calc_wave_calib(det_lamp_on, bad_pix_mask, xcal, ycal, 
+            err = kmos_calc_wave_calib(det_lamp_on, bad_pix_mask,
                     filter_ids[i-1], lamp_config, i, unused_ifus_after[i-1], 
-                    detector_edges, lines, reflines, -1.0, &lcal, 
+                    detector_edges, lines, reflines, &lcal, 
                     &(stored_qc_ar_eff[sx]), &(stored_qc_ne_eff[sx]), fit_order,
-                    line_estimate_method);
+                    lines_estimation);
             cpl_image_delete(det_lamp_on); 
             for (j = 0; j < KMOS_IFUS_PER_DETECTOR; j++) {
                 cpl_table_delete(detector_edges[j]); 
@@ -691,14 +710,14 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     
     /* Free */
     cpl_free(angles_array) ;
-    for (i = 0; i < ne; i++) cpl_free(filter_ids[i]);
+    for (i = 0; i < next; i++) cpl_free(filter_ids[i]);
     cpl_free(filter_ids);
     cpl_bivector_delete(lines);
     
     cpl_free(stored_qc_arc_sat);
     cpl_free(stored_qc_ar_eff);
     cpl_free(stored_qc_ne_eff);
-    if (line_estimate_method == 2) cpl_table_delete(reflines); 
+    if (lines_estimation == 0) cpl_table_delete(reflines); 
     
     /* QC parameters & saving */
     cpl_msg_info(__func__, "Saving data...");
@@ -720,8 +739,8 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
 
     /* Save sub-frames */
     for (a = 0; a < nb_angles; a++) {
-        for (i = 1; i <= ne ; i++) {
-            int sx = a * ne + (i - 1);
+        for (i = 1; i <= next ; i++) {
+            int sx = a * next + (i - 1);
             /* save lcal-frame */
             kmo_dfs_save_image(stored_lcal[sx], LCAL, fn_suffix, 
                     stored_sub_headers_lcal[sx], 0./0.);
@@ -729,12 +748,12 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
             /* save detector image */
             kmo_dfs_save_image(stored_det_img[sx], DET_IMG_WAVE, fn_suffix, 
                     stored_sub_headers_det_img[sx], 0./0.);
-        } // for i = ne
+        } // for i = nxte
     } // for a angles
    
     /* Free */
     cpl_free(fn_suffix);
-    for (i = 0; i < ne * nb_angles; i++) {
+    for (i = 0; i < next * nb_angles; i++) {
         cpl_image_delete(stored_lcal[i]);
         cpl_image_delete(stored_det_img[i]);
         cpl_propertylist_delete(stored_sub_headers_lcal[i]);
@@ -765,7 +784,7 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
   @param    frameset        Set of frames with ARC_OFF and several ARC_ON
   @param    nx [out]        images x size
   @param    ny [out]        images y size
-  @param    ne [out]        frames nb of extensions
+  @param    next [out]        frames nb of extensions
   @param    exptime [out]   exposure time
   @param    non_dest_rom    Flag to identify non destructive ROM
   @param    lamp_config     ARGON / NEON / ARGON+NEON
@@ -776,10 +795,10 @@ static int kmos_wave_cal_check_inputs(
         cpl_frameset            *   frameset, 
         int                     *   nx, 
         int                     *   ny,
-        int                     *   ne,
+        int                     *   next,
         double                  *   exptime,
         int                     *   non_dest_rom,
-        enum lampConfiguration  *   lamp_config)
+        lampConfiguration       *   lamp_config)
 {
     const cpl_frame     *   frame_off ;
     const cpl_frame     *   frame_on ;
@@ -795,7 +814,7 @@ static int kmos_wave_cal_check_inputs(
     /* TODO Check Lamps TODO */
 
     /* Check Entries */
-    if (nx == NULL || ny == NULL || ne == NULL || frameset == NULL) return -1;
+    if (nx == NULL || ny == NULL || next == NULL || frameset == NULL) return -1;
 
     /* Setup lamp config */
     frame_on = kmo_dfs_get_frame(frameset, ARC_ON);
@@ -814,7 +833,7 @@ static int kmos_wave_cal_check_inputs(
         cpl_msg_info(__func__, "Arc lamp: Argon + Neon");
     } else {
         *lamp_config = -1 ;
-        cpl_warning_info(__func__, "Arc lamp: UNDEFINED");
+        cpl_msg_warning(__func__, "Arc lamp: UNDEFINED");
     }
 
     /* Check READ OUT MODE */
@@ -917,7 +936,7 @@ static int kmos_wave_cal_check_inputs(
     /* Return */
     *nx = nx_off ;
     *ny = ny_off ;
-    *ne = next_off ;
+    *next = next_off ;
     *exptime = exptime_off ;
     return 1 ;
 }
